<?php
/**
 * Copyright(c)2014, Yamato Financial Co.,Ltd. All rights reserved.
 */
// {{{ requires
require_once(MODULE_REALDIR . 'mdl_yfc_paymentapi/inc/include.php');
require_once(MDL_YFCAPI_CLASSEX_PATH . 'client_extends/SC_Mdl_YFCApi_Client_Base_Ex.php');
// }}}
/**
 * 決済モジュール 決済処理: クレジットカード
 */
class SC_Mdl_YFCApi_Client_Credit extends SC_Mdl_YFCApi_Client_Base_Ex {

    /**
     * 3Dセキュア実行を行う.
     * @param $arrOrder
     * @param $arrParam
     * @param $arrPaymentInfo
     * @return bool
     */
    function do3dTran($arrOrder, $arrParam, $arrPaymentInfo) {

        //モジュール基本クラス取得
        $objMdl =& SC_Mdl_YFCApi_Ex::getInstance();

        //決済データ確認
        if (SC_Utils_Ex::isBlank($arrOrder[MDL_YFCAPI_ORDER_COL_PAYDATA])) {
            $msg = '3Dセキュア認証遷移エラー:決済データが受注情報に見つかりませんでした.';
            $objMdl->printLog($msg);
            SC_Utils_Ex::sfDispSiteError(FREE_ERROR_MSG, "", true,
                "例外エラー<br />3Dセキュア認証遷移エラー<br />この手続きは無効となりました。<br />決済データが受注情報に見つかりませんでした。");
            SC_Response_Ex::actionExit();
        } else {
            $arrPayData = SC_Util_Mdl_YFCApi_Ex::jsonDecode2Array(($arrOrder[MDL_YFCAPI_ORDER_COL_PAYDATA]));
        }

        //取引ID確認
        if ($arrPayData['order_no'] != $arrOrder['order_id']) {
            $msg = '3Dセキュア認証遷移エラー:取引IDが一致しませんでした。(' .$arrPayData['order_no'] . ':' .  $arrOrder['order_id'] . ')';
            $objMdl->printLog($msg);
            SC_Utils_Ex::sfDispSiteError(FREE_ERROR_MSG, "", true,
                "例外エラー<br />3Dセキュア認証遷移エラー<br />この手続きは無効となりました。<br />取引IDが一致しませんでした。");
            SC_Response_Ex::actionExit();
        }

        $arrMdlSetting = $objMdl->getUserSettings();

        $server_url = $this->getApiUrl('A02');

        $arrSendKey = array(
            'function_div',
            'trader_code',
            'order_no',
            'comp_cd',
            'card_no',
            'card_exp',
            'item_price',
            'item_tax',
            'cust_cd',
            'shop_id',
            'term_cd',
            'crd_res_cd',
            'res_ve',
            'res_pa',
            'res_code',
            'three_d_inf',
            'three_d_tran_id',
            'send_dt',
            'hash_value',
            'three_d_token');

        //決済ステータスを「3Dセキュア認証中」で記録
        $arrParam['action_status'] = MDL_YFCAPI_ACTION_STATUS_3D_WAIT;
        //機能区分
        $arrParam['function_div'] = 'A02';
        //カード番号
        $arrParam['card_no'] = $arrParam['CARD_NO'];
        //有効期限
        $arrParam['card_exp'] = $arrParam['CARD_EXP'];
        //3Dトークン
        $arrParam['threeDToken'] = $arrPayData['threeDToken'];

        return $this->sendOrderRequest($server_url, $arrSendKey, $arrOrder, $arrParam, $arrPaymentInfo, $arrMdlSetting);
    }


    /**
     * 3Dセキュア実行を行う(トークン使用)
     * @param $arrOrder
     * @param $arrParam
     * @param $arrPaymentInfo
     * @return bool
     */
    function do3dTokenTran($arrOrder, $arrParam, $arrPaymentInfo) {

        $code = 'A09';

        // モジュール基本クラス取得
        $objMdl =& SC_Mdl_YFCApi_Ex::getInstance();

        // 決済データ確認
        if (SC_Utils_Ex::isBlank($arrOrder[MDL_YFCAPI_ORDER_COL_PAYDATA])) {
            $msg = '3Dセキュア認証遷移エラー:決済データが受注情報に見つかりませんでした.';
            $objMdl->printLog($msg);
            SC_Utils_Ex::sfDispSiteError(FREE_ERROR_MSG, "", true,
            "例外エラー<br />3Dセキュア認証遷移エラー<br />この手続きは無効となりました。<br />決済データが受注情報に見つかりませんでした。");
            SC_Response_Ex::actionExit();
        } else {
            $arrPayData = SC_Util_Mdl_YFCApi_Ex::jsonDecode2Array(($arrOrder[MDL_YFCAPI_ORDER_COL_PAYDATA]));
        }

        // 取引ID確認
        if ($arrPayData['order_no'] != $arrOrder['order_id']) {
            $msg = '3Dセキュア認証遷移エラー:取引IDが一致しませんでした。(' .$arrPayData['order_no'] . ':' .  $arrOrder['order_id'] . ')';
            $objMdl->printLog($msg);
            SC_Utils_Ex::sfDispSiteError(FREE_ERROR_MSG, "", true,
            "例外エラー<br />3Dセキュア認証遷移エラー<br />この手続きは無効となりました。<br />取引IDが一致しませんでした。");
            SC_Response_Ex::actionExit();
        }

        $arrMdlSetting = $objMdl->getUserSettings();

        $server_url = $this->getApiUrl($code);

        $arrSendKey = array(
                'function_div',
                'trader_code',
                'order_no',
                'comp_cd',
                'token',
                'card_exp',
                'item_price',
                'item_tax',
                'cust_cd',
                'shop_id',
                'term_cd',
                'crd_res_cd',
                'res_ve',
                'res_pa',
                'res_code',
                'three_d_inf',
                'three_d_tran_id',
                'send_dt',
                'hash_value',
                'three_d_token'
        );

        // 決済ステータスを「3Dセキュア認証中」で記録
        $arrParam['action_status'] = MDL_YFCAPI_ACTION_STATUS_3D_WAIT;

        // 機能区分
        $arrParam['function_div'] = $code;

        // 決済トークン
        $arrParam['token'] = $arrParam['TOKEN'];

        // 有効期限
        $arrParam['card_exp'] = $arrParam['CARD_EXP'];

        // 3Dトークン
        $arrParam['threeDToken'] = $arrPayData['threeDToken'];

        return $this->sendOrderRequest($server_url, $arrSendKey, $arrOrder, $arrParam, $arrPaymentInfo, $arrMdlSetting);
    }

    /**
     * 決済実行を行う.
     * @param array $arrOrder
     * @param array $arrParam
     * @param array $arrPaymentInfo
     * @return bool
     */
    function doPaymentRequest($arrOrder, $arrParam, $arrPaymentInfo) {

        //モジュール基本クラス取得
        $objMdl =& SC_Mdl_YFCApi_Ex::getInstance();
        //モジュール設定値取得
        $arrMdlSetting = $objMdl->getUserSettings();
        //決済データ設定
        if (!SC_Utils_Ex::isBlank($arrOrder[MDL_YFCAPI_ORDER_COL_PAYDATA])) {
            $arrPayData = SC_Util_Mdl_YFCApi_Ex::jsonDecode2Array(($arrOrder[MDL_YFCAPI_ORDER_COL_PAYDATA]));
            $arrOrder = array_merge($arrOrder, $arrPayData);
        }
        //API設定
        $server_url = $this->getApiUrl('A01');
        //送信キー
        $arrSendKey = array(
            'function_div',
            'trader_code',
            'device_div',
            'order_no',
            'settle_price',
            'buyer_name_kanji',
            'buyer_tel',
            'buyer_email',
            'auth_div',
            'pay_way',
            'option_service_div');

        //機能区分
        $arrParam['function_div'] = 'A01';
        //決済ステータスを「決済手続き中」で記録
        $arrParam['action_status'] = MDL_YFCAPI_ACTION_STATUS_WAIT;
        //オプションサービス区分 00：通常受注 01：オプションサービス受注
        //(条件)
        //オプションサービス契約済み ：use_option==0(必須)
        //購入時カード預かり         ：register_card==1
        //預かりカードでの購入       ：use_registed_card==1
        //予約商品購入               ：tpl_is_reserve==true
        if($arrMdlSetting['use_option'] == '0' && ($arrParam['register_card'] == '1' || $arrParam['use_registed_card'] == '1' || $arrParam['tpl_is_reserve'] == true)) {
            $arrParam['option_service_div'] = '01';
        } else {
            $arrParam['option_service_div'] = '00';
        }
        //支払方法設定：本人認証サービス(3Dセキュア) 0:利用しない 1:利用する
        $arrParam['auth_div'] = ($arrPaymentInfo['TdFlag'] == '0') ? '2' : '1';
        //3Dセキュア未加入・迂回時(3Dセキュア未加入/デバイスPC[デバイスコード:2]以外の場合)
        if($arrParam['info_use_threeD'] == MDL_YFCAPI_3D_EXCLUDED
           || SC_Util_Mdl_YFCApi_Ex::getDeviceCode() != 2) {
            $arrParam['auth_div'] = '2';
        }

        //カード情報
        //オプションサービス区分が「00:通常受注」、または「01:オプションサービス受注」かつお預かりカード登録時
        if($arrParam['option_service_div'] == '00'
         || ($arrParam['option_service_div'] == '01' && $arrParam['register_card'] =='1')){
            $arrSendKey[] = 'card_code_api';                //カード会社コード(API用)
            $arrSendKey[] = 'card_no';                      //カード番号
            $arrSendKey[] = 'card_owner';                   //カード名義人
            $arrSendKey[] = 'card_exp';                     //カード有効期限
        }
        //セキュリティコード
        //認証区分が「2：３Ｄセキュアなし、セキュリティコード認証あり」
        if($arrParam['auth_div'] == '2') {
            $arrSendKey[] = 'security_code';                //セキュリティコード
        }
        //加盟店ECサイトURL
        //認証区分が「1：３Ｄセキュアあり、セキュリティコード認証なし」
        if($arrParam['auth_div'] == '1') {
            $arrSendKey[] = 'trader_ec_url';                //加盟点ECサイトURL
            $arrParam['trader_ec_url'] = SC_Utils_Ex::sfRmDupSlash(MDL_YFCAPI_TRADER_URL);
            //決済ステータスを「3Dセキュア認証中」で記録
            $arrParam['action_status'] = MDL_YFCAPI_ACTION_STATUS_3D_WAIT;
        }
        //オプションサービス区分が「01:オプションサービス受注」
        if($arrParam['option_service_div'] == '01') {
            $arrSendKey[] = 'member_id';                     //会員ＩＤ
            $arrSendKey[] = 'authentication_key';            //認証キー
            $arrSendKey[] = 'check_sum';                     //チェックサム
            $arrParam['member_id'] = SC_Util_Mdl_YFCApi_Ex::getMemberId($arrOrder['customer_id']);
            $arrParam['authentication_key'] = SC_Util_Mdl_YFCApi_Ex::getAuthenticationKey($arrOrder['customer_id']);
            $arrParam['check_sum'] = SC_Util_Mdl_YFCApi_Ex::getCheckSum($arrParam, $arrMdlSetting);
        }
        //オプションサービスで登録されているクレジットカード情報を利用
        if($arrParam['option_service_div'] == '01'
        && $arrParam['use_registed_card'] =='1'){
            $arrSendKey[] = 'card_key';                      //カード識別キー
            $arrSendKey[] = 'last_credit_date';              //最終利用日時
            //最終利用日時をセット
            $arrParam['lastCreditDate'] = $this->lfGetLastCreditDate($arrOrder['customer_id'],$arrParam['card_key']);
        }
        //予約商品購入の場合
        if($arrParam['option_service_div'] == '01'
         && $arrParam['tpl_is_reserve'] == true){
            $arrSendKey[] = 'scheduled_shipping_date';       //出荷予定日
            //出荷予定日取得
            $scheduled_shipping_date = SC_Util_Mdl_YFCApi_Ex::getMaxScheduledShippingDate($arrOrder['order_id']);
            //パラメータ用に整形
            $arrParam['scheduled_shipping_date'] = SC_Util_Mdl_YFCApi_Ex::getFormatedDate($scheduled_shipping_date);
        }

        return $this->sendOrderRequest($server_url, $arrSendKey, $arrOrder, $arrParam, $arrPaymentInfo, $arrMdlSetting);
    }

    /**
     * 決済実行を行う(トークン使用)
     * @param array $arrOrder
     * @param array $arrParam
     * @param array $arrPaymentInfo
     * @return bool
     */
    function doPaymentTokenRequest($arrOrder, $arrParam, $arrPaymentInfo) {

        $code = 'A08';

        // モジュール基本クラス取得
        $objMdl =& SC_Mdl_YFCApi_Ex::getInstance();

        // モジュール設定値取得
        $arrMdlSetting = $objMdl->getUserSettings();

        // 決済データ設定
        if (!SC_Utils_Ex::isBlank($arrOrder[MDL_YFCAPI_ORDER_COL_PAYDATA])) {
            $arrPayData = SC_Util_Mdl_YFCApi_Ex::jsonDecode2Array(($arrOrder[MDL_YFCAPI_ORDER_COL_PAYDATA]));
            $arrOrder = array_merge($arrOrder, $arrPayData);
        }

        // API設定
        $server_url = $this->getApiUrl($code);

        // 送信キー
        $arrSendKey = array(
                'function_div',
                'trader_code',
                'device_div',
                'order_no',
                'settle_price',
                'buyer_name_kanji',
                'buyer_tel',
                'buyer_email',
                'pay_way',
                'option_service_div',
                'card_code_api',
                'token',
                'ec_cart_code',
                // x3ds2_login_typeはEC-CUBEに保持していない
                // x3ds2_login_dateはEC-CUBEに保持していない
                'x3ds2_account_indicator', // dtb_customer.create_dateより判定
                // x3ds2_account_change_dateはEC-CUBEに保持していない
                // x3ds2_account_change_indicatorはEC-CUBEに保持していない
                'x3ds2_account_create_date', // dtb_customer.create_dateより判定
                // x3ds2_password_change_dateはEC-CUBEに保持していない
                // x3ds2_password_change_indicatorはEC-CUBEに保持していない
                'x3ds2_purchase_count', // dtb_orderのレコード数
                // x3ds2_card_registration_countはEC-CUBEに保持していない
                // x3ds2_activity_count_dayはEC-CUBEに保持していない
                // x3ds2_activity_count_yearはEC-CUBEに保持していない
                // x3ds2_card_registration_dateはEC-CUBEに保持していない
                // x3ds2_card_registration_indicatorはEC-CUBEに保持していない
                // x3ds2_ship_address_use_dateはEC-CUBEに保持していない
                // x3ds2_ship_address_use_indicatorはEC-CUBEに保持していない
                // x3ds2_ship_name_indicatorは半角英数字指定のため送れない
                // x3ds2_suspicious_acc_activityはEC-CUBEに保持していない
                'x3ds2_address_match', // dtb_orderとdtb_shippingを比較して設定
                // x3ds2_bill_address_cityはEC-CUBEに保持していない
                // x3ds2_bill_address_countryはEC-CUBEに保持していない
                'x3ds2_bill_address_line1', // dtb_order.order_addr01 DB上はvarchar(255)だが画面バリデーションは32文字を上限としているためカットはしない
                'x3ds2_bill_address_line2', // dtb_order.order_addr02 DB上はvarchar(255)だが画面バリデーションは32文字を上限としているためカットはしない
                // x3ds2_ship_address_line3はEC-CUBEに保持していない
                'x3ds2_bill_address_post_code', // dtb_order.order_zip01 . dtb_order.order_zip02
                // 'x3ds2_bill_address_state', // x3ds2_bill_address_countryが設定できないので送らない
                'x3ds2_email_address', // dtb_order.order_email
                // x3ds2_home_phone_cc EC-CUBEの電話番号は自宅かどうか不明なので送信しない
                // x3ds2_home_phone_subscriber EC-CUBEの電話番号は自宅かどうか不明なので送信しない
                // x3ds2_mobile_phone_cc EC-CUBEの電話番号は自宅かどうか不明なので送信しない
                // x3ds2_mobile_phone_subscriber EC-CUBEの電話番号は自宅かどうか不明なので送信しない
                // x3ds2_ship_address_cityはEC-CUBEに保持していない
                // x3ds2_ship_address_countryはEC-CUBEに保持していない
                'x3ds2_ship_address_line1', // dtb_shipping.shipping_addr01 請求先情報（住所1）と同じ設定ルール
                'x3ds2_ship_address_line2', // dtb_shipping.shipping_addr02 請求先情報（住所2）と同じ設定ルール
                // x3ds2_ship_address_line3はEC-CUBEに保持していない
                'x3ds2_ship_address_post_code', // dtb_shipping.shipping_zip01 . dtb_shipping.shipping_zip02
                // 'x3ds2_ship_address_state', // x3ds2_ship_address_countryが設定できないので送らない
                // x3ds2_work_phone_ccはEC-CUBEに保持していない
                // x3ds2_work_phone_subscriberはEC-CUBEに保持していない
                // x3ds2_instalment_payment_dataはEC-CUBEに保持していない
                // x3ds2_delivery_email_addressはEC-CUBEに保持していない
                // x3ds2_delivery_timeframeはEC-CUBEに保持していない
                // x3ds2_gift_card_amountはEC-CUBEにc保持していない
                // x3ds2_gift_card_countはEC-CUBEに保持していない
                // x3ds2_gift_card_currency_codeはEC-CUBEに保持していない
                // x3ds2_pre_order_dateは予約商品出荷予定日のためif内でキーも追加する
                'x3ds2_pre_order_purchase_indicator', // 通常商品「01」予約商品「02」
                'x3ds2_reorder_indicator', // dtb_order_itemに同一product_idの履歴があれば「02」なければ「01」
                'x3ds2_ship_address_first_use_date', // dtb_shippingを確認して出荷先が請求先と異なれば「03」同一ならば「01」
                'x3ds2_payment_date', // dtb_order.create_date
                'x3ds2_transaction_type' // 「01」固定
        );

        //支払方法設定：本人認証サービス(3Dセキュア) 0:利用しない 1:利用する
        $arrParam['auth_div'] = ($arrPaymentInfo['TdFlag'] == '0') ? '2' : '3';
        //3Dセキュア未加入・迂回時(3Dセキュア未加入/デバイスPC[デバイスコード:2]orSP[デバイスコード:1]以外の場合)
        $deviceCode = SC_Util_Mdl_YFCApi_Ex::getDeviceCode();
        if($arrParam['info_use_threeD'] == MDL_YFCAPI_3D_EXCLUDED || ($deviceCode != 2 && $deviceCode != 1)) {
            $arrParam['auth_div'] = '2';
        }

        // 機能区分
        $arrParam['function_div'] = $code;

        // 決済ステータスを「決済手続き中」で記録
        $arrParam['action_status'] = MDL_YFCAPI_ACTION_STATUS_WAIT;

        //オプションサービス区分 00：通常受注 01：オプションサービス受注
        //(条件)
        //オプションサービス契約済み ：use_option==0(必須)
        //購入時カード預かり         ：register_card==1
        //預かりカードでの購入       ：use_registed_card==1
        //予約商品購入               ：tpl_is_reserve==true
        if($arrMdlSetting['use_option'] == '0' && ($arrParam['register_card'] == '1' || $arrParam['use_registed_card'] == '1' || $arrParam['tpl_is_reserve'] == true)) {
            $arrParam['option_service_div'] = '01';
        } else {
            $arrParam['option_service_div'] = '00';
        }

        // 加盟店ECサイトURL
        // 認証区分が「2：３Ｄセキュアなし」
        if($arrParam['auth_div'] != '2') {
            $arrSendKey[] = 'trader_ec_url';
            $arrParam['trader_ec_url'] = SC_Utils_Ex::sfRmDupSlash(MDL_YFCAPI_TRADER_URL);

            //決済ステータスを「3Dセキュア認証中」で記録
            $arrParam['action_status'] = MDL_YFCAPI_ACTION_STATUS_3D_WAIT;
        }

        // カード会社コード
        // トークン決済の場合はカード会社コード9:VISAで固定
        $arrParam['card_code_api'] = '9';

        // 決済トークン
        $arrParam['token'] = $arrParam['webcollectToken'];

        // ECカートコード
        $arrParam['ec_cart_code'] = 'EC-CUBE2';

        //予約商品購入の場合
        if($arrParam['option_service_div'] == '01' && $arrParam['tpl_is_reserve'] == true){
            $arrSendKey[] = 'scheduled_shipping_date';       //出荷予定日
            //出荷予定日取得
            $scheduled_shipping_date = SC_Util_Mdl_YFCApi_Ex::getMaxScheduledShippingDate($arrOrder['order_id']);
            //パラメータ用に整形
            $arrParam['scheduled_shipping_date'] = SC_Util_Mdl_YFCApi_Ex::getFormatedDate($scheduled_shipping_date);

            // 3DS2.0加盟店送信項目
            $arrSendKey[] = 'x3ds2_pre_order_date'; // 予約商品出荷予定日
            $arrParam['x3ds2_pre_order_date'] = $arrParam['scheduled_shipping_date']; // 予約商品出荷予定日
            $arrParam['x3ds2_pre_order_purchase_indicator'] = '02'; // 予約商品「02」
        } else {
            $arrParam['x3ds2_pre_order_purchase_indicator'] = '01'; // 通常商品「01」
        }

        $this->setX3ds2($arrOrder, $arrParam);

        return $this->sendOrderRequest($server_url, $arrSendKey, $arrOrder, $arrParam, $arrPaymentInfo, $arrMdlSetting);
    }

    /**
     * 使用カードの最終利用日取得
     * @param integer $customer_id
     * @param integer $card_key
     * @return string $last_credit_date
     */
    function lfGetLastCreditDate($customer_id,$card_key){
        //モジュール基本クラス取得
        $objMdl =& SC_Mdl_YFCApi_Ex::getInstance();
        //お預かり情報照会
        $objClient = new SC_Mdl_YFCApi_Client_Util_Ex();
        $result = $objClient->doGetCard($customer_id);
        if(!$result) {
            $objMdl->printLog($objClient->getError());
            SC_Utils_Ex::sfDispSiteError(FREE_ERROR_MSG, "", true,
                "例外エラー<br />お預かり照会エラー<br />この手続きは無効となりました。<br />");
            SC_Response_Ex::actionExit();
        }
        $arrResults = $this->lfGetArrCardInfo($objClient->getResults());
        return $arrResults['cardData'][($card_key - 1)]['lastCreditDate'];
    }

    /**
     * カード預かり情報取得（整理済）
     *
     * 預かり情報１件の場合と２件以上の場合で配列の構造を合わせる
     * @param array $arrCardInfos
     * @return array $arrResults
     */
    function lfGetArrCardInfo($arrCardInfos = array()) {
        if (isset($arrCardInfos['cardUnit']) && $arrCardInfos['cardUnit'] == '1') {
            $arrTmp = array();
            foreach ($arrCardInfos as $key => $val) {
                if($key == 'cardData') {
                    $arrTmp[$key][0] = $val;
                } else {
                    $arrTmp[$key] = $val;
                }
            }
            $arrResults = $arrTmp;
        } else {
            $arrResults = $arrCardInfos;
        }

        return $arrResults;
    }

    /**
     * アカウントの保有時間を取得する.
     *
     * @param DateTime
     * @return string 02：30日未満／03：30～60日／04：60日を超える期間
     */
    private function getX3ds2AccountIndicator($createDate)
    {
        $days30before = new DateTime();
        $days30before->add(DateInterval::createFromDateString('-30 days'));
        $days60before = new DateTime();
        $days60before->add(DateInterval::createFromDateString('-60 days'));
        if ($days30before < $createDate) {
            return '02';
        } else if ($days60before < $createDate) {
            return '03';
        }
        return '04';
    }

    /**
     * 3DS2.0加盟店送信項目を設定する.
     *
     * @param array $arrOrder 注文情報
     * @param array $listParam 設定先の配列
     * @return void
     */
    private function setX3ds2($arrOrder, &$listParam)
    {
        $arrCustomer = $this->selectCustomers($arrOrder['customer_id'])[0];
        $arrShipping = $this->selectShippings($arrOrder['order_id'])[0];

        if ($arrCustomer)
        {
            // ログイン済みの場合のみ設定可能な項目
            $createDate = DateTime::createFromFormat('Y-m-d H:i:s', $arrCustomer['create_date']);
            if ($createDate == false) {
                // 変換に失敗したらミリ秒がついている前提で試す
                $createDate = DateTime::createFromFormat('Y-m-d H:i:s.v', $arrCustomer['create_date']);
            }
            if ($createDate == false) {
                // 変換に失敗したらマイクロ秒がついている前提で試す
                $createDate = DateTime::createFromFormat('Y-m-d H:i:s.u', $arrCustomer['create_date']);
            }
            $createDate->setTimezone(new DateTimeZone('UTC'));
            $listParam['x3ds2_account_indicator'] = $this->getX3ds2AccountIndicator($createDate); // dtb_customer.create_dateより判定
            $listParam['x3ds2_account_create_date'] = $createDate->format('Ymd'); // dtb_customer.create_dateより判定

            $listParam['x3ds2_purchase_count'] = $this->getX3ds2PurchaseCount($arrOrder['customer_id']); // 過去6か月間のdtb_order数を設定
            $listParam['x3ds2_reorder_indicator'] = $this->getX3ds2ReorderIndicator($arrOrder['customer_id'], $arrOrder['order_id']); // dtb_order_detailに同一product_idの履歴があれば「02」なければ「01」
        }

        $listParam['x3ds2_bill_address_line1'] = $arrOrder['order_addr01']; // dtb_order.order_addr01 DB上はvarchar(255)だが画面バリデーションは32文字を上限としているためカットはしない
        $listParam['x3ds2_bill_address_line2'] = $arrOrder['order_addr02']; // dtb_order.order_addr02 DB上はvarchar(255)だが画面バリデーションは32文字を上限としているためカットはしない
        $listParam['x3ds2_bill_address_post_code'] = $arrOrder['order_zip01'] . $arrOrder['order_zip02']; // dtb_order.order_zip01 . dtb_order.order_zip02

        $listParam['x3ds2_email_address'] = $arrOrder['order_email']; // dtb_order.order_email

        $listParam['x3ds2_ship_address_line1'] = $arrShipping['shipping_addr01']; // dtb_shipping.shipping_addr01 請求先情報（住所1）と同じ設定ルール
        $listParam['x3ds2_ship_address_line2'] = $arrShipping['shipping_addr02']; // dtb_shipping.shipping_addr02 請求先情報（住所2）と同じ設定ルール
        $listParam['x3ds2_ship_address_post_code'] = $arrShipping['shipping_zip01'] . $arrShipping['shipping_zip02']; // dtb_shipping.shipping_zip01 . dtb_shipping.shipping_zip02

        // dtb_orderとdtb_shippingを比較して設定
        if (($listParam['x3ds2_bill_address_line1'] == $listParam['x3ds2_ship_address_line1'])
            && ($listParam['x3ds2_bill_address_line2'] == $listParam['x3ds2_ship_address_line2'])
        ) {
            $listParam['x3ds2_address_match'] = '01';
            $listParam['x3ds2_ship_address_first_use_date'] = '01';
        } else {
            $listParam['x3ds2_address_match'] = '02';
            $listParam['x3ds2_ship_address_first_use_date'] = '03';
        }

        // 以下の値はメソッド外にて他の予約関係データと一緒に設定する
        // x3ds2_pre_order_date
        // x3ds2_pre_order_purchase_indicator

        $paymentDate = DateTime::createFromFormat('Y-m-d H:i:s', $arrOrder['create_date']);
        if ($paymentDate == false) {
            // 変換に失敗したらミリ秒がついている前提で試す
            $paymentDate = DateTime::createFromFormat('Y-m-d H:i:s.v', $arrOrder['create_date']);
        }
        if ($paymentDate == false) {
            // 変換に失敗したらマイクロ秒がついている前提で試す
            $paymentDate = DateTime::createFromFormat('Y-m-d H:i:s.u', $arrOrder['create_date']);
        }
        $paymentDate->setTimezone(new DateTimeZone('UTC'));
        $listParam['x3ds2_payment_date'] = $paymentDate->format('YmdHis');  // dtb_order.create_date
        $listParam['x3ds2_transaction_type'] = '01'; // 「01」固定
    }

    /**
     * create_dateを取得する.
     *
     * @param string $customer_id
     * @return dtb_customer.create_date
     */
    function selectCustomers($customer_id) {
        $objQuery = SC_Query_Ex::getSingletonInstance();
        $arrVal = array();
        $arrVal[] = $customer_id;
        return $objQuery->select(
            'create_date',
            'dtb_customer',
            'customer_id = ?',
            $arrVal
        );
    }

    /**
     * products_idを検索する.
     *
     * @param string $order_id
     * @return 指定されたdtb_orderに紐づくproduct_id
     */
    function selectProducts($order_id) {
        $objQuery = SC_Query_Ex::getSingletonInstance();
        $arrVal = array();
        $arrVal[] = $order_id;
        return $objQuery->select(
            'product_id',
            'dtb_order_detail',
            'order_id = ?',
            $arrVal
        );
    }

    /**
     * お届け先を検索する.
     *
     * @param string $order_id
     * @return dtb_shippingの配列
     */
    function selectShippings($order_id) {
        $objQuery = SC_Query_Ex::getSingletonInstance();
        $arrVal = array();
        $arrVal[] = $order_id;
        $objQuery->setOrder('shipping_id');
        // 3Dセキュア2.0で利用する項目としては第1番目の配送先だけでよい
        $arrRet = $objQuery->select(
            'shipping_zip01, shipping_zip02, shipping_addr01, shipping_addr02',
            'dtb_shipping',
            'order_id = ?',
            $arrVal
        );
        $objQuery->setOrder('');

        return $arrRet;
    }
    
    /**
     * 購入回数（全決済手段）を取得する.
     *
     * @param Customer $customer_id
     * @return integer システム日時から6か月未満の間の購入回数
     */
    function getX3ds2PurchaseCount($customer_id)
    {
        $searchStartDate = new DateTime();
        $searchStartDate = $searchStartDate
            ->add(DateInterval::createFromDateString('-6 months'));

        $objQuery = SC_Query_Ex::getSingletonInstance();
        $arrVal = array();
        $arrVal[] = $customer_id;
        $arrVal[] = $searchStartDate->format('Y-m-d H:i:s'); // DateTimeのまま比較できない
        $arrResult = $objQuery->select(
            'order_id',
            'dtb_order',
            'customer_id = ? and ? < create_date',
            $arrVal
        );

        return count($arrResult);
    }

    /**
     * 再注文区分を取得する.
     *
     * @param string $customer_id
     * @param string $order_id
     * @return string 第1引数が第2引数の商品のうち1つでも購入した履歴がある場合02、ない場合01
     */
    function getX3ds2ReorderIndicator($customer_id, $order_id)
    {
        // SQLを書けばはやいがMySQL/PostgreSQLの差を吸収するためORMを使う
        // 「購入」状態をorder_statusで判断
        $objQuery = SC_Query_Ex::getSingletonInstance();
        $arrVal = array();
        $arrVal[] = $customer_id;
        $arrVal[] = $order_id;
        $arrVal[] = '3'; // キャンセル
        $arrVal[] = '7'; // 決済処理中
        $arrOrder = $objQuery->select(
            'order_id',
            'dtb_order',
            'customer_id = ? and order_id <> ? and status not in (?, ?)',
            $arrVal
        );

        if (!$arrOrder) {
            return '01';
        }

        $OrderDetails = $this->selectProducts($order_id);
        foreach ($arrOrder as $o) {
            foreach ($OrderDetails as $od) {
                $arrVal = array();
                $arrVal[] = $o['order_id'];
                $arrVal[] = $od['product_id'];
                $arrOrderDetails = $objQuery->select(
                    'order_detail_id',
                    'dtb_order_detail',
                    'order_id = ? and product_id = ?',
                    $arrVal
                );

                if (count($arrOrderDetails) > 0) {
                    return '02';
                }
            }
        }

        return '01';
    }
}
